/*
           Martha Edburg Retirement Countdown
   2021-11-21
   v0.0.5

   See "Nearly-Autonomous Retirement Countdown Display" at Instructables.com for details.
   Calcultes and displays time to retirement.



*/


//  - - - G L O B A L S - - -  \\

const long MERsecs = 147286800L; //24,08,31,17
long totDays = 0; //using in the calcDays function
byte totYrs; //to calc total years from 2020
long totSecs; //to calc total secconds
long secsToDisp; //number to convert to string for display on the LCD
String toDisp; // converted number to a string


// array starts at 0, start at 1 for January
int days[13] = {0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
// array starts at 0, Monday is 1
String dayOfWeek[8] = {"", " Monday", " Tuesday", "Wednesday", "Thursday", " Friday", " Saturday", " Sunday"};

#include <LiquidCrystal.h>
const byte rs = 7, en = 6, d4 = 5, d5 = 4, d6 = 3, d7 = 2; //define pins
LiquidCrystal lcd(rs, en, d4, d5, d6, d7); //sets LCD to 4-bit mode, and only Write

#include <SPI.h> //for RTC
const byte SPI_SS = 10;  //set SPI SlaveSelect pin

//for reading the RTC
byte curSec; //current seconds, etc, etc, etc.
byte curMin;
byte curHr;
byte curDay;
byte curDate;
byte curMonth;
byte curYr;


void setup() {             // / /  S E T U P  \ \ \\ 
  pinMode(SPI_SS, OUTPUT);
  delay(100);
  digitalWrite(SPI_SS, HIGH); //active low, so init high
  SPI.begin();
  SPI.beginTransaction(SPISettings(100000, MSBFIRST, SPI_MODE1)); //Setup SPI. Could run faster, no need.

  lcd.begin(20, 4); //init LCD
  lcd.clear(); //clear the LCD

  if (false) { //true to set date/time
    // to set the time
    digitalWrite(SPI_SS, LOW); //select RTC
    delay(100);

    SPI.transfer(0x80); // start-address for write, then auto-inc

    SPI.transfer(decToBcd(0));  //sec, 00-59
    SPI.transfer(decToBcd(41));  //min, 00-59
    SPI.transfer(decToBcd(20)); //hour and set to 24 hr mode, do not set to more than 23
    SPI.transfer(decToBcd(7));  //day, 1-7, 1 is Monday
    SPI.transfer(decToBcd(5));  //date, 01-31
    SPI.transfer(decToBcd(12)); //month, 01-12
    SPI.transfer(decToBcd(21)); //year, 00-99

    digitalWrite(SPI_SS, HIGH); //unselect
  }

  initDisp(); //splash screen
} //end setup


void loop() {              // / /  L O O P  \ \ \\ 

  readRTC();

  //calc # of secs in full years
  totYrs = curYr - 20; //2020 is year zero, for this purpose. RTC returns 00-99 for the year.
  totSecs = (totYrs * 31536000L);  // for non-leap years
  totSecs += 86400L;  // for extra day in leap year in 2020

  calcDays();  //calc number of days (so far) in current year
  totSecs += (totDays * 86400L); //convert days to secs, add to total seconds
  totSecs += (long(curHr) * 3600L); //convert hours to secs, add to total seconds
  totSecs += (long(curMin) * 60L); //convert minutes to secs, add to total seconds
  totSecs += long(curSec); //and finally, add current seconds to total

  secsToDisp = MERsecs - totSecs; //calc remaining time to retire

  checkRetire(); //time's up?  if not:

  // disaply everything
  toDisp = numToStr(secsToDisp, 0); //seconds
  lcd.setCursor(0, 0); // col, row
  lcd.print(toDisp);

  toDisp = numToStr(secsToDisp / 60L, 1); //minutes
  lcd.setCursor(0, 1);
  lcd.print(toDisp);

  toDisp = numToStr(secsToDisp / 3600L, 2); //hours
  lcd.setCursor(0, 2);
  lcd.print(toDisp);

  toDisp = numToStr(secsToDisp / 86400L, 3); //days
  lcd.setCursor(0, 3);
  lcd.print(toDisp);

  delay(318); //no hurry, wait a little while
} // end loop



//=====functions===methods===procedures===subroutines===subprograms=====\\

void calcDays() { //calculates number of days so far in current year.
  totDays = days[curMonth]; //lookup days so far this year, up to previous month
  totDays += curDate; //and then add the current date

  if (curYr == 24 && curMonth > 2) {
    totDays += 1; //leap
  }
  if (curYr == 28 && curMonth > 2) {
    totDays += 1; //leap
  }
  totDays -= 1;  //do not count current date, only add full days in year so far
}// end calcDays


void readRTC() { //read the Real-Time Clock
  digitalWrite(SPI_SS, LOW); //select the RTC

  //read them registers
  SPI.transfer(0x00); // write start address for read, then it auto-increments
  curSec = bcdToDec(SPI.transfer(0)); //begin reading
  curMin = bcdToDec(SPI.transfer(0));
  curHr = bcdToDec(SPI.transfer(0));
  curDay = bcdToDec(SPI.transfer(0));
  curDate = bcdToDec(SPI.transfer(0));
  curMonth = bcdToDec(SPI.transfer(0));
  curYr = bcdToDec(SPI.transfer(0));

  digitalWrite(SPI_SS, HIGH); //unselect
}// end readRTC


String numToStr(long numb, byte type) {
  /*
    takes a long number, converts to a String, adds commas,
    returns formatted string for disply on the LCD.
  */
  String outString; //string to Return
  const byte dispWidth = 20;  //LCD character width
  String inString = String(numb); //convert the number to a string
  int j = (dispWidth - 1) - 8; //index in number string to display
  byte digCnt = 0;  //count digits for inserting commas
  const byte numLngth = (inString.length()); //number of times to loop

  switch (type) { //init strings, with proper spacing and time units
    case 0:
      outString = "             Seconds"; //those spaces blank out previous line displayed
      break;
    case 1:
      outString = "or           Minutes";
      break;
    case 2:
      outString = "or           Hours  ";
      break;
    case 3:
      outString = "or           Days   ";
      break;
  }
  for (int i = (numLngth - 1); i >= 0; i--) {  //i is index into the number, sorry for the bad grammer :)
    outString.setCharAt(j, inString[i]);
    j--;
    digCnt++;
    if ((digCnt == 3 && numLngth > 3) || (digCnt == 6 && numLngth > 6)) { //insert comma every 3,6 digits if more than 4 or 7
      outString.setCharAt(j, ','); //insert the comma
      j--; //move to next postion in output string
    }
  }
  return outString;
} //end numToStr


void initDisp(void) { // splash screen

  //display MERC name
  lcd.clear();
  delay(1000);
  lcd.setCursor(5, 0);
  lcd.print("Martha");
  delay(200);
  lcd.setCursor(5, 1);
  lcd.print("Edburg");
  delay(800);
  lcd.setCursor(5, 2);
  lcd.print("Retirement");
  delay(800);
  lcd.setCursor(5, 3);
  lcd.print("Countdown");
  delay(3000);

  //simple animation
  lcd.setCursor(6, 0); lcd.print("          "); delay(500);
  lcd.setCursor(6, 1); lcd.print("          "); delay(500);
  lcd.setCursor(6, 2); lcd.print("          "); delay(500);
  lcd.setCursor(6, 3); lcd.print("          "); delay(1000);

  lcd.setCursor(5, 1); lcd.print(" ");
  lcd.setCursor(7, 0); lcd.print("E"); delay(500);
  lcd.setCursor(5, 2); lcd.print(" ");
  lcd.setCursor(9, 0); lcd.print("R"); delay(500);
  lcd.setCursor(5, 3); lcd.print(" ");
  lcd.setCursor(11, 0); lcd.print("C");

  delay(2000);

  lcd.setCursor(6, 2); lcd.print("v0.0.5"); //display version 5
  delay(2000);
  lcd.clear();
  delay(500);

  //display RTC date and time
  lcd.setCursor(1, 0);
  lcd.print("Current Date/Time:");
  readRTC(); //get current date/time

  lcd.setCursor(5, 2);
  lcd.print("20"); lcd.print(curYr); lcd.print("-");
  if (curMonth < 10) { //leading zero
    lcd.print(0);
  }
  lcd.print(curMonth);
  lcd.print("-");
  if (curDate  < 10) { //leading zero
    lcd.print(0);
  }
  lcd.print(curDate);

  lcd.setCursor(2, 3);
  lcd.print(dayOfWeek[curDay]); lcd.print("  ");

  if (curHr < 10) { //leading zero
    lcd.print(0);
  }
  lcd.print(curHr);

  lcd.print(":");
  if (curMin < 10) { //leading zero
    lcd.print(0);
  }
  lcd.print(curMin);
  delay(5000);
  lcd.clear();
  delay(500);

  lcd.setCursor(7, 0);
  lcd.print("Time"); delay(500);
  lcd.setCursor(7, 1);
  lcd.print("until"); delay(500);
  lcd.setCursor(4, 2);
  lcd.print("Retirement:");

  delay(2000);
} //initDisp

void checkRetire(void) {
  if (secsToDisp <= 0) { //done!
    // loop in here forever. could be improved, more exciting.
    lcd.clear();
    delay(1000);
    lcd.setCursor(0, 0); lcd.print(" CONGRATULATIONS!!!");
    lcd.setCursor(0, 2); lcd.print("       HAPPY");
    lcd.setCursor(0, 3); lcd.print("    RETIREMENT!!!");

    while (true) { //do forever
      delay(500);
      lcd.noDisplay();
      delay(500);
      lcd.display();
    }
  }
} //

byte bcdToDec(byte val)
{
  return ((val / 16 * 10) + (val % 16));
}

byte decToBcd(byte val)
{
  return ((val / 10 * 16) + (val % 10));
}
